@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2017
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -112,25 +112,25 @@ public:
    template <typename ObjectRef>
    struct composite_cursor {
       typedef typename deref<ObjectRef>::type Object;
-      static const int
-         total= list_length<typename object_traits<Object>::elements>::value,
-         ignored= list_accumulate_unary<list_count, ignore_in_composite, typename object_traits<Object>::elements>::value;
-      static const bool
-         top_level=!mtagged_list_extract<Options, SeparatorChar>::is_specified,
-         compress= ignored>0 && total-ignored<=1;
-      static const char
+      static constexpr int
+         total = list_length<typename object_traits<Object>::elements>::value,
+         ignored = list_accumulate_unary<list_count, ignore_in_composite, typename object_traits<Object>::elements>::value;
+      static constexpr bool
+         top_level = !mtagged_list_extract<Options, SeparatorChar>::is_specified,
+         compress = ignored > 0 && total-ignored <= 1;
+      static constexpr char
          null_char = '\0',
          opening = top_level || compress ? null_char : '(',
          closing = top_level || compress ? null_char : ')',
-         sep= compress ? null_char : object_traits<Object>::IO_ends_with_eol ? '\n' : ' ';
+         sep = compress ? null_char : object_traits<Object>::IO_ends_with_eol ? '\n' : ' ';
 
-      typedef typename mtagged_list_replace<
+      using cursor_options = typename mtagged_list_replace<
                  typename mtagged_list_remove<Options, SparseRepresentation>::type,
                  OpeningBracket<char_constant<opening>>,
                  ClosingBracket<char_constant<closing>>,
                  SeparatorChar<char_constant<sep>> >::type
-         cursor_options;
-      typedef PlainPrinterCompositeCursor<cursor_options, Traits> type;
+         ;
+      using type = PlainPrinterCompositeCursor<cursor_options, Traits>;
    };
 
    template <typename Object>
@@ -200,19 +200,21 @@ using std::ends;
 template <typename Options, typename Traits>
 class PlainPrinterCompositeCursor
    : public PlainPrinter<Options, Traits> {
-   typedef PlainPrinter<Options, Traits> super;
+   using base_t = PlainPrinter<Options, Traits>;
 protected:
    char pending_sep;
    int width;
-   static const char
+   static constexpr char
       null_char = '\0',
       sep = tagged_list_extract_integral<Options, SeparatorChar>(null_char),
       opening = tagged_list_extract_integral<Options, OpeningBracket>(null_char),
       closing = tagged_list_extract_integral<Options, ClosingBracket>(null_char);
 
 public:
-   PlainPrinterCompositeCursor(typename super::ostream& os_arg, bool no_opening_by_width=false)
-      : super(os_arg), pending_sep(0), width(os_arg.width())
+   PlainPrinterCompositeCursor(typename base_t::ostream& os_arg, bool no_opening_by_width=false)
+      : base_t(os_arg)
+      , pending_sep(null_char)
+      , width(os_arg.width())
    {
       if (opening) {
          if (width) {
@@ -227,9 +229,12 @@ public:
    template <typename T>
    PlainPrinterCompositeCursor& operator<< (const T& x)
    {
-      if (pending_sep) *this->os << pending_sep;
+      if (pending_sep) {
+         *this->os << pending_sep;
+         pending_sep = null_char;
+      }
       if (width) *this->os << std::setw(width);
-      static_cast<super&>(*this) << x;
+      static_cast<base_t&>(*this) << x;
       if (sep=='\n') {
          if (! object_traits<T>::IO_ends_with_eol) *this->os << sep;
       } else {
@@ -251,39 +256,44 @@ public:
          *this->os << closing;
          if (sep=='\n') *this->os << sep;
       }
-      pending_sep=0;
+      pending_sep=null_char;
    }
 };
 
 template <typename Options, typename Traits>
 class PlainPrinterSparseCursor
    : public PlainPrinterCompositeCursor<Options, Traits> {
-   typedef PlainPrinterCompositeCursor<Options, Traits> super;
+   using base_t = PlainPrinterCompositeCursor<Options, Traits>;
 protected:
    int next_index, dim;
 
 public:
-   PlainPrinterSparseCursor(typename super::ostream& os_arg, int dim_arg)
-      : super(os_arg,true), next_index(0), dim(dim_arg)
+   PlainPrinterSparseCursor(typename base_t::ostream& os_arg, int dim_arg)
+      : base_t(os_arg, true)
+      , next_index(0)
+      , dim(dim_arg)
    {
-      if (!this->width)  // print the dimension
-         static_cast<super&>(*this) << item2composite(dim);
+      if (!this->width) {
+         // print the dimension
+         *this->os << '(' << dim << ')';
+         this->pending_sep = base_t::sep;
+      }
    }
 
    template <typename Iterator>
    PlainPrinterSparseCursor& operator<< (const Iterator& x)
    {
       if (this->width) {
-         int i=x.index();
+         const int i = x.index();
          while (next_index < i) {
             *this->os << std::setw(this->width) << '.';
             ++next_index;
          }
          *this->os << std::setw(this->width);
-         static_cast<super&>(*this) << *x;
+         static_cast<base_t&>(*this) << *x;
          ++next_index;
       } else {
-         static_cast<super&>(*this) << reinterpret_cast<const indexed_pair<Iterator>&>(x);
+         static_cast<base_t&>(*this) << reinterpret_cast<const indexed_pair<Iterator>&>(x);
       }
       return *this;
    }
@@ -296,7 +306,7 @@ public:
             ++next_index;
          }
       } else {
-         super::finish();
+         base_t::finish();
       }
    }
 };
@@ -306,17 +316,22 @@ template <typename Options> class PlainParserCompositeCursor;
 
 class PlainParserCommon {
 protected:
-   mutable std::istream* is;
+   std::istream* is;
    char *saved_egptr;
 
-   PlainParserCommon(const PlainParserCommon& other)
-      : is(other.is), saved_egptr(other.saved_egptr)
+   PlainParserCommon(const PlainParserCommon& other) = delete;
+
+   PlainParserCommon(PlainParserCommon&& other)
+      : is(other.is)
+      , saved_egptr(other.saved_egptr)
    {
-      other.is=NULL;
+      other.is = nullptr;
    }
 
    explicit PlainParserCommon(std::istream& is_arg)
-      : is(&is_arg), saved_egptr(0) { }
+      : is(&is_arg)
+      , saved_egptr(nullptr) { }
+
 public:
    ~PlainParserCommon()
    {
@@ -385,29 +400,28 @@ class PlainParser
    , public GenericInputImpl< PlainParser<Options> >
    , public GenericIOoptions< PlainParser<Options>, Options > {
 public:
-   PlainParser(std::istream& is_arg) : PlainParserCommon(is_arg) { }
+   explicit PlainParser(std::istream& is_arg) : PlainParserCommon(is_arg) {}
+   PlainParser(PlainParser&&) = default;
 
    template <typename ObjectRef>
    struct list_cursor {
-      typedef PlainParserListCursor<typename deref<ObjectRef>::type::value_type,
-                                    typename PlainPrinter<Options>::template list_cursor<ObjectRef>::cursor_options>
-         type;
+      using type = PlainParserListCursor<typename deref<ObjectRef>::type::value_type,
+                                         typename PlainPrinter<Options>::template list_cursor<ObjectRef>::cursor_options>;
    };
 
    template <typename ObjectRef>
    struct composite_cursor {
-      typedef PlainParserCompositeCursor<typename PlainPrinter<Options>::template composite_cursor<ObjectRef>::cursor_options>
-         type;
+      using type = PlainParserCompositeCursor<typename PlainPrinter<Options>::template composite_cursor<ObjectRef>::cursor_options>;
    };
 
    template <typename Object>
-   typename list_cursor<Object>::type begin_list(Object*) const
+   decltype(auto) begin_list(Object*) const
    {
       return typename list_cursor<Object>::type(*this->is);
    }
 
    template <typename Object>
-   typename composite_cursor<Object>::type begin_composite(Object*) const
+   decltype(auto) begin_composite(Object*) const
    {
       return typename composite_cursor<Object>::type(*this->is);
    }
@@ -468,13 +482,17 @@ protected:
 
 public:
    PlainParserCursor(std::istream& is_arg)
-      : PlainParserCommon(is_arg), start_pos(0)
+      : PlainParserCommon(is_arg)
+      , start_pos(nullptr)
    {
       if (is_temp)
-         start_pos=save_read_pos();
-      if (opening) set_range(opening, closing);
+         start_pos = save_read_pos();
+      if (opening)
+         set_range(opening, closing);
    }
 
+   PlainParserCursor(PlainParserCursor&&) = default;
+
    ~PlainParserCursor()
    {
       if (is_temp)
@@ -483,7 +501,8 @@ public:
 
    void finish()
    {
-      if (closing) discard_range(closing);
+      if (closing)
+         discard_range(closing);
    }
 
    bool at_end()
@@ -494,13 +513,13 @@ public:
 
 template <typename Options>
 class PlainParserCompositeCursor
-   : public PlainParserCursor<Options>,
-     public GenericInputImpl< PlainParserCompositeCursor<Options> >,
-     public GenericIOoptions< PlainParserCompositeCursor<Options>, Options > {
-   typedef PlainParserCursor<Options> super;
+   : public PlainParserCursor<Options>
+   , public GenericInputImpl< PlainParserCompositeCursor<Options> >
+   , public GenericIOoptions< PlainParserCompositeCursor<Options>, Options > {
+   using base_t = PlainParserCursor<Options>;
 public:
    PlainParserCompositeCursor(std::istream& is_arg)
-      : super(is_arg) { }
+      : base_t(is_arg) { }
 
    template <typename Data>
    void operator>> (Data& data)
@@ -511,16 +530,16 @@ public:
 
 template <typename Value, typename Options>
 class PlainParserListCursor
-   : public PlainParserCursor<Options>,
-     public GenericInputImpl< PlainParserListCursor<Value, Options> >,
-     public GenericIOoptions< PlainParserListCursor<Value, Options>, Options, 1 > {
-   typedef PlainParserCursor<Options> super;
+   : public PlainParserCursor<Options>
+   , public GenericInputImpl< PlainParserListCursor<Value, Options> >
+   , public GenericIOoptions< PlainParserListCursor<Value, Options>, Options, 1 > {
+   using base_t = PlainParserCursor<Options>;
 
    template <typename, typename> friend class PlainParserListCursor;
 
    static const bool has_sparse_representation=tagged_list_extract_integral<Options, SparseRepresentation>(false);
 protected:
-   int _size;
+   int size_;
    char* pair_egptr;
 
    int size(is_scalar)
@@ -538,7 +557,7 @@ protected:
       typedef typename PlainPrinter<Options>::template list_cursor<Value> defs;
       return defs::opening
              ? this->count_braced(defs::opening, defs::closing) :
-             super::opening
+             base_t::opening
              ? this->count_lines()
              : this->count_all_lines();
    }
@@ -548,28 +567,32 @@ protected:
       typedef typename PlainPrinter<Options>::template composite_cursor<Value> defs;
       return defs::opening
              ? this->count_braced(defs::opening, defs::closing) :
-             super::separator=='\n'
-             ? super::opening
+             base_t::separator=='\n'
+             ? base_t::opening
                ? this->count_lines()
                : this->count_all_lines()
              : this->count_words();
    }
 
 public:
-   typedef Value value_type;
+   using value_type = Value;
 
-   PlainParserListCursor(std::istream& is_arg)
-      : super(is_arg), _size(-1), pair_egptr(0)
+   explicit PlainParserListCursor(std::istream& is_arg)
+      : base_t(is_arg)
+      , size_(-1)
+      , pair_egptr(0)
    {
-      if (!super::opening && object_traits<Value>::total_dimension==0)
+      if (!base_t::opening && object_traits<Value>::total_dimension==0)
          this->set_range(0, '\n');
    }
 
+   PlainParserListCursor(PlainParserListCursor&&) = default;
+
    int size()
    {
-      if (_size<0)
-         _size=size(typename object_traits<Value>::model());
-      return _size;
+      if (size_<0)
+         size_=size(typename object_traits<Value>::model());
+      return size_;
    }
 
 protected:
@@ -596,7 +619,7 @@ protected:
 
    bool detect_sparse_representation()
    {
-      constexpr int own_missing=missing_parens(typename object_traits<Value>::model());
+      constexpr int own_missing = missing_parens(typename object_traits<Value>::model());
       constexpr int expect_open_parens = composite_depth<Value>::value - own_missing + 1;
       if (this->count_leading('(') == expect_open_parens)
       {
@@ -605,16 +628,18 @@ protected:
       return false;
    }
 public:
-   int index()
+   int index(const int index_bound)
    {
       if (!ignore_in_composite<Value>::value) {
          if (has_sparse_representation)
-            pair_egptr=this->set_temp_range('(', ')');
+            pair_egptr = this->set_temp_range('(', ')');
          else
             this->is->setstate(std::ios::failbit);
       }
-      int i=-1;
+      int i = -1;
       *this->is >> i;
+      if (!this->get_option(TrustedValue<std::true_type>()) && (i < 0 || i >= index_bound))
+         this->is->setstate(std::ios::failbit);
       return i;
    }
 
@@ -625,12 +650,14 @@ public:
              : detect_sparse_representation();
    }
 
+   bool is_ordered() const { return true; }
+
    int cols(bool tell_size_if_dense)
    {
-      return this->sub_parser().set_option(LookForward<std::true_type>()).begin_list((Value*)0).lookup_dim(tell_size_if_dense);
+      return this->sub_parser().set_option(LookForward<std::true_type>()).begin_list((Value*)0).get_dim(tell_size_if_dense);
    }
 
-   int lookup_dim(bool tell_size_if_dense)
+   int get_dim(bool tell_size_if_dense)
    {
       return (!ignore_in_composite<Value>::value && sparse_representation())
              ? this->set_option(SparseRepresentation<std::true_type>()).get_dim() :
@@ -641,14 +668,14 @@ public:
 private:
    int get_dim()
    {
-      int d=index();
+      int d = index(std::numeric_limits<int>::max());
       if (PlainParserCommon::at_end()) {
          this->discard_temp_range(')', pair_egptr);
       } else {
-         d=-1;  // it's not the dimension but the index of the first entry
+         d = -1;  // it's not the dimension but the index of the first entry
          this->skip_temp_range(pair_egptr);
       }
-      pair_egptr=NULL;
+      pair_egptr = nullptr;
       return d;
    }
 
@@ -657,7 +684,7 @@ private:
    {
       if (has_sparse_representation) {
          this->discard_temp_range(')', pair_egptr);
-         pair_egptr=NULL;
+         pair_egptr = nullptr;
       }
    }
 
@@ -675,9 +702,9 @@ public:
    {
       if (has_sparse_representation && pair_egptr) {
          this->skip_temp_range(pair_egptr);
-         pair_egptr=NULL;
+         pair_egptr = nullptr;
       } else {
-         super::skip_item();
+         base_t::skip_item();
       }
    }
 };
