@@ -1,4 +1,4 @@
-#  Copyright (c) 1997-2015
+#  Copyright (c) 1997-2018
 #  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
 #  http://www.polymake.org
 #
@@ -15,6 +15,7 @@
 
 use strict;
 use namespaces;
+use warnings qw(FATAL void syntax misc);
 
 package Polymake::Core::Extension;
 
@@ -38,14 +39,15 @@ use Polymake::Struct (
    [ '$version' => 'undef' ],       # version as string
    [ '$version_num' => 'undef' ],   # version as comparable v-string
    [ '$credit' => 'undef' ],        # Credit credit note, if present in metadata
-   [ '$short_name' => 'undef'],
-   [ '$is_bundled' => '#3' ],       # boolean
+   [ '$short_name' => '#3' ],
+   [ '$is_bundled' => 'defined(#3)' ],    # boolean
+   [ '$build_dir' => 'undef' ],     # for standalone externsions: where to find architecture-dependent files (configuration, shared modules...)
+   [ '$meta_tm' => 'undef' ],       # timestamp of the meta-file
    '$is_active',                    # boolean: configured, included in @active
    '@requires',                     # ( Extension ) direct and indirect prerequisites
    '@requires_opt',                 # ( Extension ) direct and indirect optional prerequisites
    '@replaces',                     # URIs obsoleted by this Extension
    '@conflicts',                    # URIs conflicting with this Extension
-   [ '$VCS' => 'undef' ],           # version control system for source files
    '$untrusted',                    # TRUE if comes from a writable location, that is, may be under development
    '$configured_at',                # timestamp of the last configuration for the current architecture
 );
@@ -55,6 +57,7 @@ sub new {
 
    if (-f $self->dir."/polymake.ext") {
       $self->untrusted= -w _ && (!$self->is_bundled || $DeveloperMode);
+      $self->meta_tm=(stat _)[9];
       open my $meta, $self->dir."/polymake.ext"
         or die "can't read extension description ", $self->dir, "/polymake.ext: $!\n";
       local $/;
@@ -64,19 +67,23 @@ sub new {
       my %sections=@sections;
       if (defined (my $URI=delete $sections{URI})) {
          if ($self->is_bundled) {
-            die "Bundled extensions have implicit URIs.\nURI section is not allowed in the extension description file ", $self->dir, "/polymake.ext\n";
+            die "Bundled extensions have implicit URIs.\n",
+                "URI section is not allowed in the extension description file ", $self->dir, "/polymake.ext\n";
          }
          $URI =~ s/^\s+//s;  $URI =~ s/\s+$//s;
          process_URI($self, $URI);
       } elsif (!$self->is_bundled && $self->URI ne "private:") {
          die "extension description file ", $self->dir, "/polymake.ext lacks mandatory URI section\n";
       }
-	
-      if (defined (my $short_name=delete $sections{NAME})) {
-		 $short_name =~ s/^\s+//s;  $short_name =~ s/\s+$//s;
-         $self->short_name=$short_name;
-      }
 
+      if (defined (my $short_name=delete $sections{NAME})) {
+         if ($self->is_bundled) {
+            die "Bundled extensions have fixed names derived from their top directory.\n",
+                "NAME section is not allowed in the extension description file ", $self->dir, "/polymake.ext\n";
+         }
+         $short_name =~ s/^\s+//s;  $short_name =~ s/\s+$//s;
+         $self->short_name = $short_name;
+      }
 
       if (defined (my $requires=delete $sections{REQUIRE})) {
          @{$self->requires} = $requires =~ /(\S+)/g;
@@ -97,10 +104,14 @@ sub new {
          $credit =~ s/\A(?:[ \t]*\n)*//s;
          $credit =~ s/(?<=\n)(?:[ \t]*\n)*\Z//s;
          $credit =~ s/^[ \t]*/  /gm;
-         if (defined($self->short_name)) {
-         	$credit .= "\n  ".$self->URI;
+         if ($self->is_bundled) {
+            $self->credit = new Rule::Credit($self->short_name, $credit);
+         } else {
+            if (defined($self->short_name)) {
+               $credit .= "\n  ".$self->URI."\n";
+            }
+            $self->credit=new Credit($self->short_name // $self->URI, $credit);
          }
-         $self->credit=new Credit($self, $credit);
       }
       if (defined (my $replaces=delete $sections{REPLACE})) {
          @{$self->replaces} = $replaces =~ /(\S+)/g;
@@ -119,25 +130,25 @@ sub new {
 
    } elsif (-f $self->dir."/URI") {
       open my $U, $self->dir."/URI";
-      my $URI=<$U>;
+      my $URI = <$U>;
       process_URI($self, $URI);
       if (-w $self->dir) {
+         $self->untrusted = true;
          require Polymake::Core::InteractiveCommands;
          write_initial_description($self);
-         $self->untrusted=1;
       }
 
    } elsif ($self->URI eq "private:") {
-      $self->untrusted=1;
+      $self->untrusted = true;
 
    } else {
       if (defined($self->URI)) {
          die "The extension ", $self->dir, " does not have any URI while ", $self->URI, " was expected\n";
       } else {
-         $self->URI="file://".$self->dir;
+         $self->URI = "file://".$self->dir;
+         $self->untrusted = true;
          require Polymake::Core::InteractiveCommands;
          write_initial_description($self);
-         $self->untrusted=1;
       }
    }
 
@@ -146,76 +157,75 @@ sub new {
 
 # private:
 sub process_URI {
-   my ($self, $URI)=@_;
+   my ($self, $URI) = @_;
    chomp $URI;
    if ($URI =~ s/\#([\d.]+)$//) {
-      $self->version=$1;
-      $self->version_num=eval "v$1";
+      $self->version = $1;
+      $self->version_num = eval "v$1";
    }
-   $self->URI=$URI;
+   $self->URI = $URI;
    delete $refused{$URI};
 }
 ######################################################################################
 # load all configured bundled and imported extensions
 sub init {
-   # create objects for bundled extensions, filter out unconfigured ones
-   my (@pending, %unconfigured, %seen);
-   foreach (glob("$InstallTop/bundled/*")) {
-      m{/bundled/($id_re)$}o;
-      my $conf_file="$InstallArch/$&/conf.make";
-      $seen{"bundled:$1"}=1;
-      my $ext=new($_[0], $_, "bundled:$1", 1);
-      if (-f $conf_file) {
-         push @pending, $ext;
-         $ext->configured_at=max((stat _)[9], $Application::configured_at);
-      } else {
-         $unconfigured{$ext->URI}=$ext;
-      }
-   }
-
-   # order the bundled extensions according to dependencies
-   while (@pending) {
-      my $pending=$#pending;
-      for (my $i=$pending; $i>=0; --$i) {
-         my $ext=$pending[$i];
-         my $satisfied=1;
-         my @prereqs=map { $registered_by_URI{$_} // ($satisfied=0) }
-                         @{$ext->requires},
-                         grep { $seen{$_} && !exists $unconfigured{$_} } @{$ext->requires_opt};
-         if ($satisfied) {
-            $registered_by_URI{$_}=$ext for $ext->URI, @{$ext->replaces};
-            $registered_by_dir{$ext->dir}=$ext;
-            @{$ext->requires}=uniq(map { ($_, @{$_->requires}) } @prereqs) if @prereqs;
-            push @active, $ext;
-            $ext->VCS=$CoreVCS;
-            $ext->is_active=1;
-            splice @pending, $i, 1;
+   # create objects for bundled extensions
+ LOAD:
+   {
+      # extensions are ordered by inter-dependencies, prerequisites coming first
+      foreach my $name (@BundledExts) {
+         my $ext = new(__PACKAGE__, "$InstallTop/bundled/$name", "bundled:$name", $name);
+         if ($DeveloperMode && $ext->meta_tm > $ConfigTime) {
+            warn_print("meta-file of bundled extension $name has been changed since last configuration;\nPerforming automatic reconfiguration, please be patient...\n");
+            my $build_opt = $ENV{POLYMAKE_BUILD_ROOT} && "--build $ENV{POLYMAKE_BUILD_ROOT}";
+            my $config_log = `cd $InstallTop; $^X support/configure.pl $build_opt 2>&1`;
+            if ($?) {
+               die "Automatic reconfiguration failed, the complete log including the error diagnostics is shown below.\n",
+                  "Please investigate the reasons and rerun the configure script manually, possibly with different options.\n\n",
+                  $config_log;
+            }
+            my %ConfigFlags = load_config_file("$InstallArch/config.ninja", $InstallTop);
+            $ConfigTime = (stat "$InstallArch/config.ninja")[9];
+            @BundledExts = $ConfigFlags{BundledExts} =~ /(\S+)/g;
+            @active = ();
+            redo LOAD;
          }
-      }
-      if ($pending==$#pending) {
-         # no further progress
-         warn_print( "Bundled extension", $pending>1 && "s", " ", join(", ", map { $_->dir =~ $filename_re } @pending),
-                     " could not be loaded because of missing or cyclic dependencies.\n",
-                     "Please investigate the reason and repeat the configuration step and/or revise the REQUIRE/REQUIRE_OPT sections." );
-         last;
+         $registered_by_URI{$_} = $ext for $ext->URI, @{$ext->replaces};
+         $registered_by_dir{$ext->dir} = $ext;
+         $ext->configured_at = max($ConfigTime, $Application::configured_at);
+         my @bad;
+         my @prereqs = map { $registered_by_URI{$_} // do { push @bad, $_; () } } @{$ext->requires};
+         if (@bad) {
+            die "Corrupted configuration of bundled extensions: prerequisites @bad appear after the dependent extension $name or are disabled.\n",
+                "Please investigate and re-run the configure script.\n";
+         }
+         push @prereqs, grep { defined } map { $registered_by_URI{$_} } @{$ext->requires_opt};
+         @{$ext->requires}=uniq(map { ($_, @{$_->requires}) } @prereqs) if @prereqs;
+         push @active, $ext;
+         $ext->is_active = true;
       }
    }
 
    # register the inactive bundled extensions in order to recognize them as prerequisites of other extensions and rulefiles
-   $registered_by_URI{$_->URI}=$_ for (@pending, values %unconfigured);
+   foreach my $bundled_dir (glob("$InstallTop/bundled/*")) {
+      unless (exists $registered_by_dir{$bundled_dir}) {
+         my ($name) = $bundled_dir =~ $filename_re;
+         my $ext = new(__PACKAGE__, $bundled_dir, "bundled:$name", $name);
+         $registered_by_URI{$ext->URI} = $ext;
+      }
+   }
 
-   $num_bundled=@active;
+   $num_bundled = @active;
 
    # perform dependency checks on standalone extensions
-
-   @pending=map { new($_[0],$_) } @User::extensions;
+   my @pending = map { new(__PACKAGE__, $_) } @User::extensions;
    my ($list_updated, @survived, %failed);
 
-   my $rounds=0;
+   my $rounds = 0;
    while (@pending) {
       my @next_round;
       foreach my $ext (@pending) {
-         if (defined (my $other=$registered_by_URI{$ext->URI})) {
+         if (defined(my $other = $registered_by_URI{$ext->URI})) {
             if ($other->URI eq $ext->URI) {
                warn_print( "Extensions ", $other->dir, " and ", $ext->dir, " have identical URI: ", $ext->URI, <<'.' );
 
@@ -230,14 +240,14 @@ and assign distinct URIs.
                            "The first one is therefore removed from your settings.\n",
                            "If this is in error, revise the REPLACE sections in the extension description files" );
             }
-            $list_updated=1;
+            $list_updated = true;
             next;
          }
 
-         my $conflicting=$conflicts{$ext->URI};
+         my $conflicting = $conflicts{$ext->URI};
          unless ($conflicting) {
             foreach (@{$ext->conflicts}) {
-               $conflicting=$registered_by_URI{$_} and last;
+               $conflicting = $registered_by_URI{$_} and last;
             }
          }
          if ($conflicting) {
@@ -246,25 +256,26 @@ and assign distinct URIs.
                         "The first one is therefore removed from your settings.\n",
                         "If this is in error, revise the CONFLICT sections in the extension description files" );
 
-            $failed{$_}="conflicts with other extensions" for $ext->URI, @{$ext->replaces};
-            $list_updated=1;
+            $failed{$_} = "conflicts with other extensions" for $ext->URI, @{$ext->replaces};
+            $list_updated = true;
             next;
          }
 
          my @prereqs;
-         my $satisfied=1;
+         my $satisfied = true;
          for (@{$ext->requires}) {
-            my $prereq_version= s/\#([\d.]+)$// && $1;
-            if (defined (my $prereq=$registered_by_URI{$_})) {
+            my $prereq_version = s/\#([\d.]+)$// && $1;
+            if (defined(my $prereq = $registered_by_URI{$_})) {
                if (length($prereq_version)) {
                   if ($prereq->is_bundled) {
                      warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
                                  "imposes a forbidden version requirement on a bundled extension ", $prereq->dir, "\n",
                                  "Please revise the REQUIRE section in the description file ", $ext->dir."/polymake.ext\n",
                                  "and re-import the extension.\n" );
-                     $failed{$_}="wrong dependencies" for $ext->URI, @{$ext->replaces};
-                     $list_updated=1;
-                     $satisfied=0;  last;
+                     $failed{$_} = "wrong dependencies" for $ext->URI, @{$ext->replaces};
+                     $list_updated = true;
+                     $satisfied = false;
+                     last;
 
                   } elsif ($prereq->URI ne $_) {
                      warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
@@ -286,52 +297,55 @@ and assign distinct URIs.
                                  "If this is in error, revise the REQUIRE section in the extension description file\n",
                                  "or try to upgrade the second one to a newer version if available and then re-import the first one" );
 
-                     $failed{$_}="unsatisfied versioned dependencies" for $ext->URI, @{$ext->replaces};
-                     $list_updated=1;
-                     $satisfied=0;  last;
+                     $failed{$_} = "unsatisfied versioned dependencies" for $ext->URI, @{$ext->replaces};
+                     $list_updated = true;
+                     $satisfied = false;
+                     last;
                   }
                }
-	       push @prereqs, $prereq, @{$prereq->requires};
+               push @prereqs, $prereq, @{$prereq->requires};
 
-            } elsif (defined (my $reason=$failed{$_})) {
+            } elsif (defined(my $reason = $failed{$_})) {
                warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
                            "depends on another extension $_ which could not be loaded due to $reason.\n",
                            "More details have been reported in the log above.\n",
                            "Re-import this extension when you have fixed the problem in the first place" );
 
-               $failed{$_}="unsatisfied dependencies" for $ext->URI, @{$ext->replaces};
-               $list_updated=1;
-               $satisfied=0;  last;
+               $failed{$_} = "unsatisfied dependencies" for $ext->URI, @{$ext->replaces};
+               $list_updated = true;
+               $satisfied = false;
+               last;
 
             } else {
                push @next_round, $ext;
-               $satisfied=0;  last;
+               $satisfied = false;
+               last;
             }
          }
          next unless $satisfied;
 
-         $registered_by_dir{$ext->dir}=$ext;
-         $registered_by_URI{$ext->URI}=$ext;
+         $registered_by_dir{$ext->dir} = $ext;
+         $registered_by_URI{$ext->URI} = $ext;
          my @old_replaces;
          foreach (@{$ext->replaces}) {
-            if (defined (my $other=$registered_by_URI{$_})) {
+            if (defined(my $other = $registered_by_URI{$_})) {
                if ($other->URI eq $_) {
                   warn_print( "Extension $_ installed at ", $other->dir, "\n",
                               "is declared as replaced by extension ", $ext->URI, " installed at ", $ext->dir, "\n",
                               "and therefore removed from your settings.\n",
                               "If this is in error, revise the REPLACE section in the extension configuration file,\n",
-			      "then re-import the removed extension." );
+                              "then re-import the removed extension." );
 
                   push @old_replaces, @{$other->replaces};
                   delete @registered_by_URI{@{$other->replaces}};
                   delete_from_list(\@survived, $other);
-                  $list_updated=1;
+                  $list_updated = true;
                }
             }
-            $registered_by_URI{$_}=$ext;
+            $registered_by_URI{$_} = $ext;
          }
 
-         if (my @forgotten_replaces=grep { !exists $registered_by_URI{$_} } @old_replaces) {
+         if (my @forgotten_replaces = grep { !exists $registered_by_URI{$_} } @old_replaces) {
             warn_print( "Extension(s) phased out as announced above provided additional compatibility URIs\n",
                         "which do not occur any more in the current extension ", $ext->dir, "\n",
                         (map { "  $_\n" } @forgotten_replaces),
@@ -341,8 +355,8 @@ and assign distinct URIs.
                         "or consider upgrading dependent extensions to newer versions, if available." );
          }
 
-         $conflicts{$_}=$ext for @{$ext->conflicts};
-         @{$ext->requires}=uniq(@prereqs) if @prereqs;
+         $conflicts{$_} = $ext for @{$ext->conflicts};
+         @{$ext->requires} = uniq(@prereqs) if @prereqs;
          push @survived, $ext;
       }
       last if @next_round == @pending;
@@ -350,15 +364,15 @@ and assign distinct URIs.
          warn_print( "Extension order in your settings \@User::extensions is incompatible with the inter-dependencies\n",
                      "between the extensions, they will be automatically reordered.\n",
                      "Please revise the results in $PrivateDir/prefer.pl after finishing the running polymake session." );
-         $list_updated=1;
+         $list_updated = true;
       }
-      @pending=@next_round;
+      @pending = @next_round;
    }
 
    if (@pending) {
-      my %would_provide=map { ($_ => 1) } map { $_->URI, @{$_->replaces} } @pending;
+      my %would_provide = map { ($_ => true) } map { $_->URI, @{$_->replaces} } @pending;
       foreach my $ext (@pending) {
-         if (my @unknown=grep { !$would_provide{$_} } @{$ext->requires}) {
+         if (my @unknown = grep { !$would_provide{$_} } @{$ext->requires}) {
             warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
                         "has unresolved prerequisite(s)\n",
                         ( map { "  $_\n" } @unknown ),
@@ -375,11 +389,11 @@ and assign distinct URIs.
                         "and re-import the needed extensions in proper order induced by justified dependencies.\n" );
          }
       }
-      $list_updated=1;
+      $list_updated = true;
    }
 
    if ($list_updated) {
-      @User::extensions=grep { exists $registered_by_dir{$_} } @User::extensions;
+      @User::extensions = grep { exists $registered_by_dir{$_} } @User::extensions;
       $Prefs->custom->set('@extensions');
    }
 
@@ -399,7 +413,8 @@ and assign distinct URIs.
                               "which has been disabled for the current architecture $Arch\n",
                               "because of failed build configuration or further dependencies.\n" );
                }
-               $ext->is_active=0;  last;
+               $ext->is_active = false;
+               last;
             }
          }
          unless ($ext->is_active &&= $ext->configure) {
@@ -421,21 +436,19 @@ and assign distinct URIs.
       delete @User::disabled_extensions{@obsolete};
       $Prefs->custom->set('%disabled_extensions');
    }
-
-   CPlusPlus::init_private_wrapper();
 }
 #######################################################################################
 sub configure {
-   my ($self, @options)=@_;
-   my $ext_dir=$self->dir;
-   my $ext_build_dir=$ext_dir;
+   my ($self, @options) = @_;
+   my $ext_dir = $self->dir;
+   my $ext_build_dir = $ext_dir;
 
-   # don't look for a Makefile in an installed extension, it can never exist
    if ($ext_build_dir =~ s{^\Q${InstallTop}\E(?=/ext/)}{$InstallArch}o) {
+      $self->build_dir = $ext_build_dir;
       if (@options == 1 && $options[0] eq "--help") {
          warn_print( "Extension $ext_dir is already built and installed, it does not need any further configuration." );
-      } elsif (-f "$ext_build_dir/conf.make") {
-         $self->configured_at=max((stat _)[9], $Application::configured_at);
+      } elsif (-f "$ext_build_dir/config.ninja") {
+         $self->configured_at = max((stat _)[9], $Application::configured_at);
          @options and warn_print( "Extension $ext_dir is already built and installed, ignoring configuration options @options." );
          1
       } else {
@@ -448,28 +461,26 @@ For the meanwhile, the extension will stay disabled for architecture $Arch.
          0
       }
    } else {
-      $ext_build_dir="$ext_dir/build.$Arch";
-      my $err;
-      if (!-f "$ext_build_dir/conf.make" || @options) {
-         require Polymake::Configure;
-         $err=Configure::configure_extension(@_);
-      } else {
-         $self->configured_at=max((stat _)[9], $Application::configured_at);
-         my $top_match;
-         if (open my $MF, "$ext_dir/Makefile") {
-            local $_;
-            while (<$MF>) {
-               if (m{^ \s* ProjectTop \s* :?= \s* \Q${InstallTop}\E \s* $ }xo) {
-                  $top_match=1;
-                  last;
+      set_build_dir($self);
+      unless (@options) {
+         if (-f (my $build_file=$self->build_dir."/build.ninja")) {
+            my $conf_tm = (stat _)[9];
+            $self->configured_at = max($conf_tm, $Application::configured_at);
+            if ($conf_tm >= $self->meta_tm and open my $bf, $build_file) {
+               local $_;
+               while (<$bf>) {
+                  # the core system configuration file is included first:
+                  # does it match the current system?
+                  if (m{^\s*include\s+(\S+)/config.ninja\s*$}) {
+                     return 1 if $1 eq $InstallArch;
+                     last;
+                  }
                }
             }
          }
-         if (!$top_match) {
-            require Polymake::Configure;
-            $err=Configure::configure_extension($self, "--repeat", $Arch);
-         }
       }
+      require Polymake::Configure;
+      my $err=Configure::configure_extension(@_);
       if (length($err)) {
          if ($err ne "silent\n") {
             warn_print( "Build configuration of extension $ext_dir failed with following error(s):\n$err\n" );
@@ -480,13 +491,23 @@ For the meanwhile, the extension will stay disabled for architecture $Arch.
       }
    }
 }
+
+sub set_build_dir {
+   my ($self) = @_;
+   # workspace polymake and workspace extension should use the same build directory names
+   if ($InstallArch =~ m{/build(?:\.[^/]+)?/\w+$}) {
+      $self->build_dir = $self->dir.$&;
+   } else {
+      $self->build_dir = $self->dir."/build/".($ENV{POLYMAKE_BUILD_MODE} || "Opt");
+   }
+}
 ########################################################################################
 sub lookup {
-   (undef, my $what)=@_;
+   (undef, my $what) = @_;
    my $by_dir;
    unless ($what =~ s{^($id_re)$}{bundled:$1}o) {
       replace_special_paths($what);
-      $by_dir=-d $what and $what=Cwd::abs_path($what);
+      $by_dir = -d $what and $what = Cwd::abs_path($what);
    }
    if ($by_dir) {
       $registered_by_dir{$what} || croak( "Extension at location $what is not registered" )
@@ -498,7 +519,7 @@ sub lookup {
 sub app_dir { $_[0]->dir."/apps/".$_[1]->name }
 
 sub versioned_URI {
-   my $self=shift;
+   my $self = shift;
    $self->URI . (defined($self->version) && "#".$self->version)
 }
 #####################################################################################
@@ -510,10 +531,10 @@ sub versioned_URI {
 # Throws an exception if $mandatory and no Extension loaded.
 
 sub provide {
-   my ($pkg, $URI, $mandatory)=@_;
-   if (defined (my $ext=$registered_by_URI{$URI})) {
+   my ($pkg, $URI, $mandatory) = @_;
+   if (defined (my $ext = $registered_by_URI{$URI})) {
       if ($ext->is_active) {
-         $ext;
+         $ext
       } elsif ($mandatory) {
          die "Extension $URI is not configured for the current architecture $Arch.\n",
              "Activate it by polymake shell command\n",
@@ -525,7 +546,7 @@ sub provide {
                      "If you need them, activate the extension by polymake shell command\n",
                      "  reconfigure_extension(\"$URI\");\n",
                      "then reload the object from the file backup copy." );
-         undef;
+         undef
       }
 
    } elsif ($URI =~ m|^file://| and -d (my $ext_dir=$')) {
@@ -540,7 +561,7 @@ sub provide {
                         "import the extension with polymake shell commmand\n",
                         "  import_extension(\"$ext_dir\");\n",
                         "then reload the object from the file backup copy." );
-            undef;
+            undef
          }
       }
 
@@ -550,85 +571,69 @@ sub provide {
       } else {
          warn_print( "Extension $URI is not known.\n",
                      "The properties defined in this extension will be missing in your object.\n" );
-         undef;
+         undef
       }
 
-   } elsif (!exists $refused{$URI} && defined ($ext=&locate_unknown)) {
-      $ext;
+   } elsif (!exists $refused{$URI} && defined ($ext = &locate_unknown)) {
+      $ext
 
    } elsif ($refused{$URI} eq "ignore") {
-      0;
+      0
 
    } elsif ($refused{$URI} eq "stop" || $mandatory) {
       die "Extension $URI is required for loading the data.\n";
 
    } else {
-      undef;
+      undef
    }
 }
 #####################################################################################
 # private:
 sub activate {
-   my ($self)=@_;
-   local $Application::extension=$self;
-   $self->is_active=1;
+   my ($self) = @_;
+   local $Application::extension = $self;
+   $self->is_active = true;
    foreach my $app_dir (glob($self->dir."/apps/*")) {
       $app_dir =~ $filename_re;
-      if (defined (my $app=lookup Application($1))) {
-	 $app->load_extension($app_dir);
+      if (defined (my $app = lookup Application($1))) {
+         $app->load_extension($app_dir);
       }
    }
    push @active, $self;
 }
 #####################################################################################
-sub get_source_VCS {
-   my $self=shift;
-   $self->VCS ||= do {
-      if ($self->is_bundled || index($self->dir, "$InstallTop/ext/")==0) {
-         die "Can't modify an installed bundled extension\n";
-      } elsif (!-w $self->dir) {
-         die "You don't have permission to create or change files in ", $self->dir, "\n";
-      } else {
-         require Polymake::SourceVersionControl;
-         new SourceVersionControl($self->dir);
-      }
+sub get_source_tree {
+   my ($self) = @_;
+   if ($self->untrusted) {
+      require Polymake::SourceTree;
+      new SourceTree($self->is_bundled ? $InstallTop : $self->dir);
+   } elsif ($self->is_bundled || index($self->dir, "$InstallTop/ext/") == 0) {
+      die "Can't modify an installed bundled extension\n";
+   } else {
+      die "You don't have permission to create or change files in ", $self->dir, "\n";
    }
 }
-#####################################################################################
-# protected:
-sub activate_configure_script {
-   my ($self)=@_;
-   if (-f $self->dir."/configure.pl.template" and my $vcs=eval { &get_source_VCS }) {
-      $vcs->rename_file("configure.pl.template", "configure.pl");
-   }
+###############################################################################################
+sub list_prerequisite_extensions {
+   my ($self) = @_;
+   join(" ", map { $_->is_bundled ? $_->short_name : $_->dir } @{$self->requires});
 }
 #####################################################################################
-package _::Credit;
+package Polymake::Core::Extension::Credit;
 
 use Polymake::Struct (
    [ '@ISA' => 'Rule::Credit' ],
-   [ '$product' => '""' ],
 );
 
-sub new {
-   my $self=&_new;
-   if ($_[0]->is_bundled) {
-      ($self->product)= $_[0]->URI =~ /:($id_re)$/o;
-   } else {
-      $self->product=defined($_[0]->short_name)?$_[0]->short_name : $_[0]->URI;
-   }
-   bless $self, "Polymake::Core::Rule::Credit";
-}
-
 sub display {
-   my $self=shift;
+   my ($self) = @_;
    dbg_print( "used extension ", $self->product, "\n", $self->text, "\n" );
-   $self->shown=1;
+   $self->shown = 1;
 }
 
 sub toFileString {
-   my $self=shift;
-   $self->file_string ||= do {
+   my ($self) = @_;
+   $self->file_string //= do {
       my ($copyright)= $self->text =~ /(copyright\b.*)/im;
       (defined($copyright) ? "\n$copyright\n" : $self->text) . $self->product . "\n"
    }
